@@ -41,6 +41,7 @@
 #include "rhd_connector.h"
 #include "rhd_output.h"
 #include "rhd_regs.h"
+#include "rhd_hdmi.h"
 #ifdef ATOM_BIOS
 #include "rhd_atombios.h"
 #include "rhd_atomout.h"
@@ -64,7 +65,11 @@ struct transmitter {
     void (*Destroy) (struct rhdOutput *Output);
     Bool (*Property) (struct rhdOutput *Output,
 		      enum rhdPropertyAction Action, enum rhdOutputProperty Property, union rhdPropertyData *val);
-
+#ifdef NOT_YET
+    Bool (*WrappedPropertyCallback) (struct rhdOutput *Output,
+		      enum rhdPropertyAction Action, enum rhdOutputProperty Property, union rhdPropertyData *val);
+    void *PropertyPrivate;
+#endif
     void *Private;
 };
 
@@ -93,6 +98,7 @@ enum encoderMode {
 };
 
 enum encoderID {
+    ENCODER_NONE,
     ENCODER_DIG1,
     ENCODER_DIG2
 };
@@ -106,6 +112,7 @@ struct DIGPrivate
     Bool Coherent;
     Bool RunDualLink;
     DisplayModePtr Mode;
+    struct rhdHdmi *Hdmi;
 
     /* LVDS */
     Bool FPDI;
@@ -746,6 +753,17 @@ ATOMTransmitterModeValid(struct rhdOutput *Output, DisplayModePtr Mode)
     return MODE_OK;
 }
 
+/*
+ *
+ */
+void
+rhdPrintDigDebug(RHDPtr rhdPtr, const char *name)
+{
+    xf86DrvMsgVerb(rhdPtr->scrnIndex, X_INFO, 7, "%s: DIGn_CNTL: n=1: 0x%x n=2: 0x%x\n",
+	   name, RHDRegRead(rhdPtr, RV620_DIG1_CNTL),
+	   RHDRegRead(rhdPtr, DIG2_OFFSET + RV620_DIG1_CNTL));
+}
+
 /*
  *
  */
@@ -763,6 +781,8 @@ ATOMTransmitterSet(struct rhdOutput *Output, struct rhdCrtc *Crtc, DisplayModePt
     atc->Coherent = Private->Coherent;
     atc->PixelClock = Mode->SynthClock;
 
+    rhdPrintDigDebug(rhdPtr,__func__);
+
     if (Private->RunDualLink) {
 	atc->Mode = atomDualLink;
 
@@ -785,6 +805,45 @@ ATOMTransmitterSet(struct rhdOutput *Output, struct rhdCrtc *Crtc, DisplayModePt
 
     rhdAtomDigTransmitterControl(rhdPtr->atomBIOS, transPrivate->atomTransmitterID,
 				 atomTransSetup, atc);
+    rhdPrintDigDebug(rhdPtr,__func__);
+}
+
+/*
+ *
+ */
+static CARD32
+digProbeEncoder(struct rhdOutput *Output)
+{
+    if (Output->Id == RHD_OUTPUT_KLDSKP_LVTMA) {
+	return ENCODER_DIG2;
+    } else {
+	Bool swap = (RHDRegRead(Output, RV620_DCIO_LINK_STEER_CNTL)
+		     & RV62_LINK_STEER_SWAP) == RV62_LINK_STEER_SWAP;
+
+	switch (Output->Id) {
+	    case RHD_OUTPUT_UNIPHYA:
+		if (swap) {
+		    RHDDebug(Output->scrnIndex, "%s: detected ENCODER_DIG2 for UNIPHYA\n",__func__);
+		    return ENCODER_DIG2;
+		} else {
+		    RHDDebug(Output->scrnIndex, "%s: detected ENCODER_DIG1 for UNIPHYA\n",__func__);
+		    return ENCODER_DIG1;
+		}
+		break;
+	    case RHD_OUTPUT_UNIPHYB:
+		if (swap) {
+		    RHDDebug(Output->scrnIndex, "%s: detected ENCODER_DIG1 for UNIPHYB\n",__func__);
+		    return ENCODER_DIG1;
+		} else {
+		    RHDDebug(Output->scrnIndex, "%s: detected ENCODER_DIG2 for UNIPHYB\n",__func__);
+		    return ENCODER_DIG2;
+		}
+		break;
+	    default:
+		return ENCODER_NONE; /* should not get here */
+	}
+    }
+    return ENCODER_NONE;
 }
 
 /*
@@ -801,6 +860,8 @@ ATOMTransmitterPower(struct rhdOutput *Output, int Power)
 
     RHDFUNC(Output);
 
+    rhdPrintDigDebug(rhdPtr,__func__);
+
     if (Private->RunDualLink)
 	atc->LinkCnt = atomDualLink;
     else
@@ -808,6 +869,27 @@ ATOMTransmitterPower(struct rhdOutput *Output, int Power)
 
     atc->Coherent = Private->Coherent;
 
+    if (atc->Encoder == atomEncoderNone) {
+	switch (digProbeEncoder(Output)) {
+	    case ENCODER_DIG1:
+		if (rhdPtr->DigEncoderOutput[0]) {
+		    RHDDebug(Output->scrnIndex,"%s: DIG1 for %s already taken\n",__func__,Output->Name);
+		    return;
+		}
+		atc->Encoder = atomEncoderDIG1;
+		break;
+	    case ENCODER_DIG2:
+		if (rhdPtr->DigEncoderOutput[1]) {
+		    RHDDebug(Output->scrnIndex,"%s: DIG2 for %s already taken\n",__func__,Output->Name);
+		    return;
+		}
+		atc->Encoder = atomEncoderDIG2;
+		break;
+	    default:
+		return;
+	}
+    }
+
     switch (Power) {
 	case RHD_POWER_ON:
 	    rhdAtomDigTransmitterControl(rhdPtr->atomBIOS, transPrivate->atomTransmitterID,
@@ -829,6 +911,7 @@ ATOMTransmitterPower(struct rhdOutput *Output, int Power)
 					 atomTransDisable, atc);
 	    break;
     }
+    rhdPrintDigDebug(rhdPtr,__func__);
 }
 
 /*
@@ -875,6 +958,8 @@ struct DIGEncoder
 {
     Bool Stored;
 
+    CARD32 StoredOff;
+
     CARD32 StoredRegExt1DiffPostDivCntl;
     CARD32 StoredRegExt2DiffPostDivCntl;
     CARD32 StoredDIGClockPattern;
@@ -909,10 +994,13 @@ static void
 LVDSEncoder(struct rhdOutput *Output)
 {
     struct DIGPrivate *Private = (struct DIGPrivate *)Output->Private;
-    CARD32 off = (Private->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+    CARD32 off;
 
     RHDFUNC(Output);
 
+    ASSERT(Private->EncoderID != ENCODER_NONE);
+
+    off = (Private->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
     /* Clock pattern ? */
     RHDRegMask(Output, off + RV620_DIG1_CLOCK_PATTERN, 0x0063, 0xFFFF);
     /* set panel type: 18/24 bit mode */
@@ -931,10 +1019,12 @@ static void
 TMDSEncoder(struct rhdOutput *Output)
 {
     struct DIGPrivate *Private = (struct DIGPrivate *)Output->Private;
-    CARD32 off = (Private->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+    CARD32 off;
 
     RHDFUNC(Output);
 
+    ASSERT(Private->EncoderID != ENCODER_NONE);
+    off = (Private->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
     /* clock pattern ? */
     RHDRegMask(Output, off + RV620_DIG1_CLOCK_PATTERN, 0x001F, 0xFFFF);
     /* color format RGB - normal color format 24bpp, Twin-Single 30bpp or Dual 48bpp*/
@@ -951,33 +1041,50 @@ static void
 EncoderSet(struct rhdOutput *Output, struct rhdCrtc *Crtc, DisplayModePtr Mode)
 {
     struct DIGPrivate *Private = (struct DIGPrivate *)Output->Private;
-    CARD32 off = (Private->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+    RHDPtr rhdPtr = RHDPTRI(Output);
+    CARD32 off;
 
     RHDFUNC(Output);
+
+    ASSERT(Private->EncoderID != ENCODER_NONE);
+    off = (Private->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+
+    rhdPrintDigDebug(rhdPtr,__func__);
+
+    RHDRegMask(Output, off + RV620_DIG1_CNTL, Output->Crtc->Id,
+	       RV62_DIG_SOURCE_SELECT);
+
     if (Output->Id == RHD_OUTPUT_UNIPHYA) {
 	/* select LinkA ?? */
-	RHDRegMask(Output, RV620_DCIO_LINK_STEER_CNTL, 0,
+	RHDRegMask(Output, RV620_DCIO_LINK_STEER_CNTL,
 		   ((Private->EncoderID == ENCODER_DIG2)
 		    ? RV62_LINK_STEER_SWAP
-		    : 0)); /* swap if DIG2 */
+		    : 0), RV62_LINK_STEER_SWAP); /* swap if DIG2 */
 	if (!Private->RunDualLink) {
-	    RHDRegMask(Output, off + RV620_DIG1_CNTL, 0, RV62_DIG_SWAP |  RV62_DIG_DUAL_LINK_ENABLE);
+	    RHDRegMask(Output, off + RV620_DIG1_CNTL,
+		       0,
+		       RV62_DIG_SWAP |RV62_DIG_DUAL_LINK_ENABLE);
 	} else {
 	    RHDRegMask(Output, off + RV620_DIG1_CNTL,
-		       ((Private->EncoderID == ENCODER_DIG2)
-			?  RV62_DIG_SWAP
-			: 0) | RV62_DIG_DUAL_LINK_ENABLE,
-		       RV62_DIG_SWAP | RV62_DIG_DUAL_LINK_ENABLE );
+		       RV62_DIG_DUAL_LINK_ENABLE,
+		       RV62_DIG_SWAP | RV62_DIG_DUAL_LINK_ENABLE);
 	}
     } else if (Output->Id == RHD_OUTPUT_UNIPHYB) {
-	RHDRegMask(Output, off + RV620_DIG1_CNTL, 0, RV62_DIG_SWAP | RV62_DIG_DUAL_LINK_ENABLE);
 	/* select LinkB ?? */
 	RHDRegMask(Output, RV620_DCIO_LINK_STEER_CNTL,
 		   ((Private->EncoderID == ENCODER_DIG2)
 		    ? 0
 		    : RV62_LINK_STEER_SWAP), RV62_LINK_STEER_SWAP);
+	if (!Private->RunDualLink)
+	    RHDRegMask(Output, off + RV620_DIG1_CNTL,
+		       0,
+		       RV62_DIG_SWAP |  RV62_DIG_DUAL_LINK_ENABLE);
+	 else
+	    RHDRegMask(Output, off + RV620_DIG1_CNTL,
+		       RV62_DIG_SWAP | RV62_DIG_DUAL_LINK_ENABLE,
+		       RV62_DIG_SWAP | RV62_DIG_DUAL_LINK_ENABLE);
     } else { /* LVTMA */
-	RHDRegMask(Output, RV620_EXT2_DIFF_POST_DIV_CNTL, 0, 0x1 << RV62_EXT2_DIFF_DRIVER_ENABLE_SHIFT);
+	RHDRegMask(Output, RV620_EXT2_DIFF_POST_DIV_CNTL, 0, RV62_EXT2_DIFF_DRIVER_ENABLE);
     }
 
     if (Private->EncoderMode == LVDS)
@@ -998,6 +1105,7 @@ EncoderSet(struct rhdOutput *Output, struct rhdCrtc *Crtc, DisplayModePtr Mode)
 	       | RV62_DIG_DUAL_LINK_ENABLE
 	       | RV62_DIG_STEREOSYNC_SELECT
 	       | RV62_DIG_SOURCE_SELECT);
+    rhdPrintDigDebug(rhdPtr,__func__);
 }
 
 /*
@@ -1007,38 +1115,74 @@ static void
 EncoderPower(struct rhdOutput *Output, int Power)
 {
     struct DIGPrivate *Private = (struct DIGPrivate *)Output->Private;
-    CARD32 off = (Private->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+    CARD32 off;
+    enum encoderID EncoderID = Private->EncoderID;
+    RHDPtr rhdPtr = RHDPTRI(Output);
 
     RHDFUNC(Output);
+
+    if (EncoderID == ENCODER_NONE) {
+	EncoderID = digProbeEncoder(Output);
+	switch (EncoderID) {
+	    case ENCODER_DIG1:
+		if (rhdPtr->DigEncoderOutput[0]) {
+		    RHDDebug(Output->scrnIndex,"%s: DIG1 for %s already taken\n",__func__,Output->Name);
+		    return;
+		}
+		break;
+	    case ENCODER_DIG2:
+		if (rhdPtr->DigEncoderOutput[1]) {
+		    RHDDebug(Output->scrnIndex,"%s: DIG2 for %s already taken\n",__func__,Output->Name);
+		    return;
+		}
+		break;
+	    default:
+		return;
+	}
+    }
+
+    off = (EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+
     /* clock src is pixel PLL */
     RHDRegMask(Output, RV620_DCCG_SYMCLK_CNTL, 0x0,
-	       0x3 << ((Private->EncoderID == ENCODER_DIG2)
+	       0x3 << ((EncoderID == ENCODER_DIG2)
 		       ? RV62_SYMCLKB_SRC_SHIFT
 		       : RV62_SYMCLKA_SRC_SHIFT));
 
+    rhdPrintDigDebug(rhdPtr,__func__);
     switch (Power) {
 	case RHD_POWER_ON:
+	    RHDDebug(Output->scrnIndex,"%s(RHD_POWER_ON, %i)\n",__func__,
+		     EncoderID);
 	    /* enable DIG */
 	    RHDRegMask(Output, off + RV620_DIG1_CNTL, 0x10, 0x10);
-	    RHDRegMask(Output, (Private->EncoderID == ENCODER_DIG2)
+	    RHDRegMask(Output, (EncoderID == ENCODER_DIG2)
 		       ? RV620_DCCG_PCLK_DIGB_CNTL
 		       : RV620_DCCG_PCLK_DIGA_CNTL,
 		       RV62_PCLK_DIGA_ON, RV62_PCLK_DIGA_ON); /* @@@ */
+	    rhdPrintDigDebug(rhdPtr,__func__);
 	    return;
 	case RHD_POWER_RESET:
 	case RHD_POWER_SHUTDOWN:
 	default:
+	    RHDDebug(Output->scrnIndex,"%s(RHD_POWER_SHUTDOWN, %i)\n",__func__,
+		     EncoderID);
 	    /* disable differential clock driver */
-	    if (Private->EncoderID == ENCODER_DIG1)
-		RHDRegMask(Output, RV620_EXT1_DIFF_POST_DIV_CNTL, 0, RV62_EXT1_DIFF_DRIVER_ENABLE);
+	    if (EncoderID == ENCODER_DIG1)
+		RHDRegMask(Output, RV620_EXT1_DIFF_POST_DIV_CNTL,
+			   0,
+			   RV62_EXT1_DIFF_DRIVER_ENABLE);
 	    else
-		RHDRegMask(Output, RV620_EXT2_DIFF_POST_DIV_CNTL, 0, 0x1 << RV62_EXT2_DIFF_DRIVER_ENABLE_SHIFT);
+		RHDRegMask(Output, RV620_EXT2_DIFF_POST_DIV_CNTL,
+			   0,
+			   RV62_EXT2_DIFF_DRIVER_ENABLE);
 	    /* disable DIG */
 	    RHDRegMask(Output, off + RV620_DIG1_CNTL, 0x0, 0x10);
-	    RHDRegMask(Output, (Private->EncoderID == ENCODER_DIG2)
+	    RHDRegMask(Output, (EncoderID == ENCODER_DIG2)
 		       ? RV620_DCCG_PCLK_DIGB_CNTL
 		       : RV620_DCCG_PCLK_DIGA_CNTL,
 		       0, RV62_PCLK_DIGA_ON); /* @@@ */
+	    rhdPrintDigDebug(rhdPtr,__func__);
 	    return;
     }
 }
@@ -1051,10 +1195,15 @@ EncoderSave(struct rhdOutput *Output)
 {
     struct DIGPrivate *digPrivate = (struct DIGPrivate *)Output->Private;
     struct DIGEncoder *Private = (struct DIGEncoder *)(digPrivate->Encoder.Private);
-    CARD32 off = (digPrivate->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+    CARD32 off;
+    enum encoderID EncoderID;
 
     RHDFUNC(Output);
 
+    EncoderID = digProbeEncoder(Output);
+    off = (EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+    Private->StoredOff = off;
+
     Private->StoredRegExt1DiffPostDivCntl          = RHDRegRead(Output, off + RV620_EXT1_DIFF_POST_DIV_CNTL);
     Private->StoredRegExt2DiffPostDivCntl          = RHDRegRead(Output, off + RV620_EXT2_DIFF_POST_DIV_CNTL);
     Private->StoredDIGClockPattern = RHDRegRead(Output, off + RV620_DIG1_CLOCK_PATTERN);
@@ -1063,13 +1212,11 @@ EncoderSave(struct rhdOutput *Output)
     Private->StoredTMDSCntl        = RHDRegRead(Output, off + RV620_TMDS1_CNTL);
     Private->StoredDCIOLinkSteerCntl = RHDRegRead(Output, RV620_DCIO_LINK_STEER_CNTL);
     Private->StoredDCCGPclkDigCntl    = RHDRegRead(Output,
-						   (digPrivate->EncoderID
-						    == ENCODER_DIG2)
+						   (off == DIG2_OFFSET)
 						   ? RV620_DCCG_PCLK_DIGB_CNTL
 						   : RV620_DCCG_PCLK_DIGA_CNTL);
     Private->StoredDCCGSymclkCntl     = RHDRegRead(Output, RV620_DCCG_SYMCLK_CNTL);
     Private->StoredBlModCntl          = RHDRegRead(Output, RV620_LVTMA_BL_MOD_CNTL);
-
     Private->Stored = TRUE;
 }
 
@@ -1081,7 +1228,7 @@ EncoderRestore(struct rhdOutput *Output)
 {
     struct DIGPrivate *digPrivate = (struct DIGPrivate *)Output->Private;
     struct DIGEncoder *Private = (struct DIGEncoder *)(digPrivate->Encoder.Private);
-    CARD32 off = (digPrivate->EncoderID == ENCODER_DIG2) ? DIG2_OFFSET : DIG1_OFFSET;
+    CARD32 off;
 
     RHDFUNC(Output);
 
@@ -1090,6 +1237,7 @@ EncoderRestore(struct rhdOutput *Output)
 		   "%s: No registers stored.\n", __func__);
 	return;
     }
+    off = Private->StoredOff;
 
     RHDRegWrite(Output, off + RV620_EXT1_DIFF_POST_DIV_CNTL, Private->StoredRegExt1DiffPostDivCntl);
     RHDRegWrite(Output, off + RV620_EXT2_DIFF_POST_DIV_CNTL, Private->StoredRegExt2DiffPostDivCntl);
@@ -1099,7 +1247,7 @@ EncoderRestore(struct rhdOutput *Output)
     RHDRegWrite(Output, off + RV620_DIG1_CLOCK_PATTERN, Private->StoredDIGClockPattern);
     RHDRegWrite(Output, off + RV620_LVDS1_DATA_CNTL, Private->StoredLVDSDataCntl);
     RHDRegWrite(Output, off + RV620_TMDS1_CNTL, Private->StoredTMDSCntl);
-    RHDRegWrite(Output, (digPrivate->EncoderID == ENCODER_DIG2)
+    RHDRegWrite(Output, (off == DIG2_OFFSET)
 		? RV620_DCCG_PCLK_DIGB_CNTL
 		: RV620_DCCG_PCLK_DIGA_CNTL,
 		Private->StoredDCCGPclkDigCntl);
@@ -1249,14 +1397,25 @@ DigPower(struct rhdOutput *Output, int Power)
     struct DIGPrivate *Private = (struct DIGPrivate *)Output->Private;
     struct transmitter *Transmitter = &Private->Transmitter;
     struct encoder *Encoder = &Private->Encoder;
+    Bool enableHDMI;
 
     RHDDebug(Output->scrnIndex, "%s(%s,%s)\n",__func__,Output->Name,
 	     rhdPowerString[Power]);
 
+    if(Output->Connector != NULL) {
+	/* check if attached monitor supports HDMI */
+	enableHDMI = RHDConnectorEnableHDMI(Output->Connector);
+	if (enableHDMI && Private->EncoderMode == TMDS_DVI)
+	    Private->EncoderMode = TMDS_HDMI;
+	else if (!enableHDMI && Private->EncoderMode == TMDS_HDMI)
+	    Private->EncoderMode = TMDS_DVI;
+    }
+
     switch (Power) {
 	case RHD_POWER_ON:
 	    Encoder->Power(Output, Power);
 	    Transmitter->Power(Output, Power);
+	    RHDHdmiEnable(Private->Hdmi, Private->EncoderMode == TMDS_HDMI);
 	    return;
 	case RHD_POWER_RESET:
 	    Transmitter->Power(Output, Power);
@@ -1266,6 +1425,7 @@ DigPower(struct rhdOutput *Output, int Power)
 	default:
 	    Transmitter->Power(Output, Power);
 	    Encoder->Power(Output, Power);
+	    RHDHdmiEnable(Private->Hdmi, FALSE);
 	    return;
     }
 }
@@ -1319,6 +1479,7 @@ DigMode(struct rhdOutput *Output, DisplayModePtr Mode)
 
     Encoder->Mode(Output, Crtc, Mode);
     Transmitter->Mode(Output, Crtc, Mode);
+    RHDHdmiSetMode(Private->Hdmi, Mode);
 }
 
 /*
@@ -1335,6 +1496,7 @@ DigSave(struct rhdOutput *Output)
 
     Encoder->Save(Output);
     Transmitter->Save(Output);
+    RHDHdmiSave(Private->Hdmi);
 }
 
 /*
@@ -1351,6 +1513,7 @@ DigRestore(struct rhdOutput *Output)
 
     Encoder->Restore(Output);
     Transmitter->Restore(Output);
+    RHDHdmiRestore(Private->Hdmi);
 }
 
 /*
@@ -1367,11 +1530,150 @@ DigDestroy(struct rhdOutput *Output)
 
     Encoder->Destroy(Output);
     Transmitter->Destroy(Output);
-
+    RHDHdmiDestroy(Private->Hdmi);
+#ifdef NOT_YET
+    if (Transmitter->PropertyPrivate)
+	RhdAtomDestroyBacklightControlProperty(Output, Transmitter->PropertyPrivate);
+#endif
     xfree(Private);
     Output->Private = NULL;
 }
 
+/*
+ *
+ */
+static Bool
+DigAllocFree(struct rhdOutput *Output, enum rhdOutputAllocation Alloc)
+{
+    struct DIGPrivate *Private = (struct DIGPrivate *)Output->Private;
+    RHDPtr rhdPtr = RHDPTRI(Output);
+    char *TransmitterName;
+
+    RHDFUNC(rhdPtr);
+
+    switch (Output->Id) {
+	case RHD_OUTPUT_KLDSKP_LVTMA:
+	    TransmitterName = "KLDSKP_LVTMA";
+	    break;
+	case RHD_OUTPUT_UNIPHYA:
+	    TransmitterName = "UNIPHYA";
+	    break;
+	case RHD_OUTPUT_UNIPHYB:
+	    TransmitterName = "UNIPHYB";
+	    break;
+	default:
+	    return FALSE;
+    }
+    switch (Alloc) {
+	case RHD_OUTPUT_ALLOC:
+
+	    if (Private->EncoderID != ENCODER_NONE)
+		return TRUE;
+
+	    /*
+	     * LVTMA can only use DIG2. Thus exclude
+	     * DIG1 for LVTMA and prefer it for the
+	     * UNIPHYs.
+	     */
+	    if (Output->Id == RHD_OUTPUT_KLDSKP_LVTMA) {
+		if (!rhdPtr->DigEncoderOutput[1]) {
+		    rhdPtr->DigEncoderOutput[1] = Output;
+		    Private->EncoderID = ENCODER_DIG2;
+		    xf86DrvMsg(Output->scrnIndex, X_INFO,
+			       "Mapping DIG2 encoder to %s\n",TransmitterName);
+		return TRUE;
+		} else
+		    return FALSE;
+	    } else {
+		struct ATOMTransmitterPrivate *transPrivate =
+		    (struct ATOMTransmitterPrivate *)Private->Transmitter.Private;
+		struct atomTransmitterConfig *atc = &transPrivate->atomTransmitterConfig;
+		if (!rhdPtr->DigEncoderOutput[0]) {
+		    rhdPtr->DigEncoderOutput[0] = Output;
+		    Private->EncoderID = ENCODER_DIG1;
+		    atc->Encoder = atomEncoderDIG1;
+		    xf86DrvMsg(Output->scrnIndex, X_INFO,
+			       "Mapping DIG1 encoder to %s\n",TransmitterName);
+		    return TRUE;
+		} else if (!rhdPtr->DigEncoderOutput[1]) {
+		    rhdPtr->DigEncoderOutput[1] = Output;
+		    Private->EncoderID = ENCODER_DIG2;
+		    atc->Encoder = atomEncoderDIG2;
+		    xf86DrvMsg(Output->scrnIndex, X_INFO,
+			       "Mapping DIG2 encoder to %s\n",TransmitterName);
+		    return TRUE;
+		} else
+		    return FALSE;
+	    }
+
+	case RHD_OUTPUT_FREE:
+		Private->EncoderID = ENCODER_NONE;
+	    if (rhdPtr->DigEncoderOutput[0] == Output) {
+		rhdPtr->DigEncoderOutput[0] = NULL;
+		return TRUE;
+	    } else if (rhdPtr->DigEncoderOutput[1] == Output) {
+		rhdPtr->DigEncoderOutput[1] = NULL;
+		return TRUE;
+	    } else
+		return FALSE;
+	    break;
+	default:
+	    return FALSE;
+    }
+}
+
+/*
+ *
+ */
+static Bool
+rhdDIGSetCoherent(RHDPtr rhdPtr,struct rhdOutput *Output)
+{
+    Bool coherent = FALSE;
+    int  from = X_CONFIG;
+
+    switch (RhdParseBooleanOption(&rhdPtr->coherent, Output->Name)) {
+	case RHD_OPTION_NOT_SET:
+	case RHD_OPTION_DEFAULT:
+	    from = X_DEFAULT;
+	    coherent = FALSE;
+	    break;
+	case RHD_OPTION_ON:
+	    coherent = TRUE;
+	    break;
+	case RHD_OPTION_OFF:
+	    coherent = FALSE;
+	    break;
+    }
+    xf86DrvMsg(rhdPtr->scrnIndex,from,"Setting %s to %scoherent\n",
+	       Output->Name,coherent ? "" : "in");
+
+    return coherent;
+}
+
+/*
+ *
+ */
+#ifdef NOT_YET
+static Bool
+digTransmitterPropertyWrapper(struct rhdOutput *Output,
+			      enum rhdPropertyAction Action,
+			      enum rhdOutputProperty Property,
+			      union rhdPropertyData *val)
+{
+    struct DIGPrivate *Private = (struct DIGPrivate *)Output->Private;
+    void *storePrivate = Output->Private;
+    Bool (*func)(struct rhdOutput *,enum rhdPropertyAction, enum rhdOutputProperty,
+		  union rhdPropertyData *) = Private->Transmitter.WrappedPropertyCallback;
+    Bool ret;
+
+    Output->Private = Private->Transmitter.PropertyPrivate;
+    ret = func(Output, Action, Property, val);
+    Output->Private = storePrivate;
+
+    return ret;
+}
+#endif
+
 /*
  *
  */
@@ -1397,17 +1699,17 @@ RHDDIGInit(RHDPtr rhdPtr,  enum rhdOutputType outputType, CARD8 ConnectorType)
     Output->Restore = DigRestore;
     Output->Destroy = DigDestroy;
     Output->Property = DigPropertyControl;
+    Output->AllocFree = DigAllocFree;
 
     Private = xnfcalloc(sizeof(struct DIGPrivate), 1);
     Output->Private = Private;
 
-    Private->Coherent = FALSE;
+    Private->EncoderID = ENCODER_NONE;
 
     switch (outputType) {
 	case RHD_OUTPUT_UNIPHYA:
 #if defined (ATOM_BIOS) && defined (ATOM_BIOS_PARSER)
 	    Output->Name = "UNIPHY_A";
-	    Private->EncoderID = ENCODER_DIG1;
 	    Private->Transmitter.Private =
 		(struct ATOMTransmitterPrivate *)xnfcalloc(sizeof (struct ATOMTransmitterPrivate), 1);
 
@@ -1423,9 +1725,9 @@ RHDDIGInit(RHDPtr rhdPtr,  enum rhdOutputType outputType, CARD8 ConnectorType)
 		struct ATOMTransmitterPrivate *transPrivate =
 		    (struct ATOMTransmitterPrivate *)Private->Transmitter.Private;
 		struct atomTransmitterConfig *atc = &transPrivate->atomTransmitterConfig;
-		atc->Coherent = Private->Coherent;
-		atc->Encoder = atomEncoderDIG1;
+		atc->Coherent = Private->Coherent = rhdDIGSetCoherent(rhdPtr, Output);
 		atc->Link = atomTransLinkA;
+		atc->Encoder = atomEncoderNone;
 		if (RHDIsIGP(rhdPtr->ChipSet)) {
 		    AtomBiosArgRec data;
 		    data.val = 1;
@@ -1453,7 +1755,6 @@ RHDDIGInit(RHDPtr rhdPtr,  enum rhdOutputType outputType, CARD8 ConnectorType)
 	case RHD_OUTPUT_UNIPHYB:
 #if defined (ATOM_BIOS) && defined (ATOM_BIOS_PARSER)
 	    Output->Name = "UNIPHY_B";
-	    Private->EncoderID = ENCODER_DIG2;
 	    Private->Transmitter.Private =
 		(struct atomTransmitterPrivate *)xnfcalloc(sizeof (struct ATOMTransmitterPrivate), 1);
 
@@ -1469,9 +1770,9 @@ RHDDIGInit(RHDPtr rhdPtr,  enum rhdOutputType outputType, CARD8 ConnectorType)
 		struct ATOMTransmitterPrivate *transPrivate =
 		    (struct ATOMTransmitterPrivate *)Private->Transmitter.Private;
 		struct atomTransmitterConfig *atc = &transPrivate->atomTransmitterConfig;
-		atc->Coherent = Private->Coherent;
-		atc->Encoder = atomEncoderDIG2;
+		atc->Coherent = Private->Coherent = rhdDIGSetCoherent(rhdPtr, Output);
 		atc->Link = atomTransLinkB;
+		atc->Encoder = atomEncoderNone;
 		if (RHDIsIGP(rhdPtr->ChipSet)) {
 		    AtomBiosArgRec data;
 		    data.val = 2;
@@ -1498,10 +1799,9 @@ RHDDIGInit(RHDPtr rhdPtr,  enum rhdOutputType outputType, CARD8 ConnectorType)
 
 	case RHD_OUTPUT_KLDSKP_LVTMA:
 	    Output->Name = "UNIPHY_KLDSKP_LVTMA";
-	    Private->EncoderID = ENCODER_DIG2;
+	    Private->Coherent = rhdDIGSetCoherent(rhdPtr, Output);
 	    Private->Transmitter.Private =
 		(struct LVTMATransmitterPrivate *)xnfcalloc(sizeof (struct LVTMATransmitterPrivate), 1);
-
 	    Private->Transmitter.Sense = NULL;
 	    Private->Transmitter.ModeValid = LVTMATransmitterModeValid;
 	    if (ConnectorType != RHD_CONNECTOR_PANEL) {
@@ -1528,6 +1828,7 @@ RHDDIGInit(RHDPtr rhdPtr,  enum rhdOutputType outputType, CARD8 ConnectorType)
 	    return NULL;
     }
 
+
     Encoder = (struct DIGEncoder *)(xnfcalloc(sizeof (struct DIGEncoder),1));
     Private->Encoder.Private = Encoder;
     Private->Encoder.ModeValid = EncoderModeValid;
@@ -1542,17 +1843,27 @@ RHDDIGInit(RHDPtr rhdPtr,  enum rhdOutputType outputType, CARD8 ConnectorType)
 	    Private->EncoderMode = LVDS;
 	    GetLVDSInfo(rhdPtr, Private);
 #ifdef ATOM_BIOS
-	    if (Private->BlLevel < 0)
-		Private->BlLevel = RhdAtomSetupBacklightControlProperty(Output, &Private->Transmitter.Property);
+#ifdef NOT_YET
+	    if (Private->BlLevel < 0) {
+		Private->BlLevel = RhdAtomSetupBacklightControlProperty(Output,
+									&Private->Transmitter.WrappedPropertyCallback,
+									&Private->Transmitter.PropertyPrivate);
+		if (Private->Transmitter.PropertyPrivate)
+		    Private->Transmitter.Property = digTransmitterPropertyWrapper;
+	    }
 #endif
+#endif
+	    Private->Hdmi = NULL;
 	    break;
 	case RHD_CONNECTOR_DVI:
 	    Private->RunDualLink = FALSE; /* will be set later acc to pxclk */
 	    Private->EncoderMode = TMDS_DVI;
+	    Private->Hdmi = RHDHdmiInit(rhdPtr, Output);
 	    break;
 	case RHD_CONNECTOR_DVI_SINGLE:
 	    Private->RunDualLink = FALSE;
-	    Private->EncoderMode = TMDS_DVI;  /* currently also HDMI */
+	    Private->EncoderMode = TMDS_DVI; /* changed later to HDMI if aplicateable */
+	    Private->Hdmi = RHDHdmiInit(rhdPtr, Output);
 	    break;
     }
 
