@@ -38,6 +38,10 @@ typedef enum rhdOutputType {
     RHD_OUTPUT_KLDSKP_LVTMA,
     RHD_OUTPUT_UNIPHYA,
     RHD_OUTPUT_UNIPHYB,
+    RHD_OUTPUT_UNIPHYC,
+    RHD_OUTPUT_UNIPHYD,
+    RHD_OUTPUT_UNIPHYE,
+    RHD_OUTPUT_UNIPHYF,
     RHD_OUTPUT_TMDSB = RHD_OUTPUT_NONE,
     RHD_OUTPUT_LVDS = RHD_OUTPUT_NONE,
     RHD_OUTPUT_LVTMB = RHD_OUTPUT_NONE
@@ -57,6 +61,11 @@ enum rhdOutputProperty {
     RHD_OUTPUT_COHERENT
 };
 
+enum rhdOutputAllocation {
+    RHD_OUTPUT_ALLOC,
+    RHD_OUTPUT_FREE
+};
+
 char *rhdPowerString[4];
 
 /*
@@ -89,6 +98,7 @@ struct rhdOutput {
     void (*Destroy) (struct rhdOutput *Output);
     Bool (*Property) (struct rhdOutput *Output,
 		      enum rhdPropertyAction Action, enum rhdOutputProperty Property, union rhdPropertyData *val);
+    Bool (*AllocFree) (struct rhdOutput *Output, enum rhdOutputAllocation Alloc);
     /* Driver Private data */
     rhdOutputDriverPrivate *OutputDriverPrivate;
     /* Output Private data */
